# Generated by Django 3.0 on 2020-02-27 09:42

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('ripple', '0002_user_phone_number'),
    ]

    operations = [
        migrations.CreateModel(
            name='FeatureOption',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=200)),
                ('description', models.CharField(blank=True, default='', max_length=200)),
                ('active', models.BooleanField(default=False)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='FeatureUrl',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('description', models.CharField(blank=True, default='', max_length=200)),
                ('regex', models.TextField(max_length=800)),
                ('feature', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='urls', to='ripple.FeatureOption')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
